/*
 *  ircd-ratbox: an advanced Internet Relay Chat Daemon(ircd).
 *  m_hacklmax.c: Allows admins to hack the server's local max users.
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *
 *  m_hacklmax.c - (C) Alan 'alz' Milford, 26th July 2003. (alz@froginternet.com)
 *
 *  Usage Details:
 * 
 *  This command is available for use by server admins only,
 * 
 *  /setlmax <number>      - Sets max current users to <number>.
 *  /resetlmax             - Sets max current users to current local user count.
 *
 *  This does not effect global operations, the module will only alter
 *  the max local client count - providing it is more than the current 
 *  user count - and less than the existing current user count.
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "hash.h"

/* NO_SND_HACKLMAX_WALLOPS
 *
 * defining this will stop ircd sending a wallops message when an 
 * the admin hacks the max user count.
 *
 */
#define NO_SND_HACKLMAX_WALLOPS

static void mo_setlmax(struct Client *, struct Client *, int, char**);
static void mo_resetlmax(struct Client *, struct Client *, int, char**);

struct Message setlmax_msgtab = {
  "SETLMAX", 0, 0, 1, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, mo_setlmax}
};

struct Message resetlmax_msgtab = {
  "RESETLMAX", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, mo_resetlmax}
};

#ifndef STATIC_MODULES

void
_modinit(void)
{
  mod_add_cmd(&setlmax_msgtab);
  mod_add_cmd(&resetlmax_msgtab);
}

void
_moddeinit(void)
{
  mod_del_cmd(&setlmax_msgtab);
  mod_del_cmd(&resetlmax_msgtab);
}

const char *_version = "1.1";
#endif

/*
** mo_setlmax
**      parv[0] = sender prefix
**      parv[1] = new usercount
*/
static void mo_setlmax(struct Client *client_p, struct Client *source_p,
                        int parc, char *parv[])
{
	int newlmax = 0;
	int oldlmax = 0;
	struct Client *target_p;
	if (!IsAdmin(source_p))
	{
		sendto_one(source_p, ":%s NOTICE %s :You are not the server admin.", me.name, parv[0]);
		return;
	}
	if ((source_p) == NULL)
	{
		return;
	}
	if (parv[1] == NULL)
	{
		sendto_one(source_p, ":%s NOTICE %s :No new max local user count specified", me.name, parv[0]);
		return;
	}
	newlmax = atoi(parv[1]);
	if (newlmax < Count.local)
	{
		sendto_one(source_p, ":%s NOTICE %s :New max less than current count (%d)", me.name, parv[0], Count.local);
		return;
	}
	if (newlmax > Count.max_loc)
	{
		sendto_one(source_p, ":%s NOTICE %s :New max more than current max (%d)", me.name, parv[0], Count.max_loc);
		return;
	}
	oldlmax = Count.max_loc;
	Count.max_loc = newlmax;
	sendto_one(source_p, ":%s NOTICE %s :[setlmax] set: %u -> %u", me.name, parv[0], oldlmax, newlmax);  
	#ifndef NO_SND_HACKLMAX_WALLOPS 
		sendto_wallops_flags(UMODE_WALLOP, &me,
			"[setlmax] [%s!%s@%s] set: %u -> %u",
			source_p->name, source_p->username, source_p->host, oldlmax, newlmax);
	#endif
}

/*
** mo_resetlmax
**      parv[0] = sender prefix
**      parv[1] = new usercount
*/
static void mo_resetlmax(struct Client *client_p, struct Client *source_p,
                        int parc, char *parv[])
{
	struct Client *target_p;
	if (!IsAdmin(source_p))
	{
		sendto_one(source_p, ":%s NOTICE %s :You are not the server admin.", me.name, parv[0]);
		return;
	}
	if ((source_p) == NULL)
	{
		return;
	}
	sendto_one(source_p, ":%s NOTICE %s :[setlmax] set: %d -> %d", me.name, parv[0], Count.max_loc, Count.local);  
	#ifndef NO_SND_HACKLMAX_WALLOPS 
		sendto_wallops_flags(UMODE_WALLOP, &me,
			"[setlmax] [%s!%s@%s] set: %d -> %d",
			source_p->name, source_p->username, source_p->host, Count.max_loc, Count.local);
	#endif
	Count.max_loc = Count.local;
}


